<?php

namespace App\Http\Controllers\Admin\Navigation;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\Navigation\TypeEnum;
use App\Repositories\Enums\Navigation\VisitTypeEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Navigation\Category;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Navigation\CategoryRepository;
use App\Repositories\Validators\Navigation\CategoryValidator;

/**
 * Class CategoriesController.
 *
 * @package namespace App\Http\Controllers\Navigation;
 */
class CategoriesController extends Controller
{
    /**
     * @var CategoryRepository
     */
    protected $repository;

    /**
     * @var CategoryValidator
     */
    protected $validator;

    /**
     * CategoriesController constructor.
     *
     * @param CategoryRepository $repository
     * @param CategoryValidator $validator
     */
    public function __construct(CategoryRepository $repository, CategoryValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));

        return Response::success($lists);
    }


    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        if (!$data['pid']) {
            $this->validate($request, [
                'logo' => 'required'
            ]);
        }

        if ($data['visit_type'] == VisitTypeEnum::COMMON) {
            $data['hidden_status'] = 0;
        }

        if ($data['visit_type'] == VisitTypeEnum::OWNED) {
            $data['hidden_status'] = 1;
        }


//        if ((int)$data['type'] === TypeEnum::ORGANIZATION) {
//            $this->validate($request, [
//                'organization_id' => 'required|integer'
//            ]);
//        }

        try {
            $data['slug'] = Str::random(6);
            $data['admin_id'] = optional(login_admin())->id;
            $data['tier'] = Category::byIdGetTier($data['pid']);
            $data['status'] = ModelStatusEnum::OK;
            $department = $this->repository->create($data);
            return Response::success($department);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $department = $this->repository->find($id);
        return Response::success($department);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));

        if ($data['visit_type'] == VisitTypeEnum::COMMON) {
            $data['hidden_status'] = 0;
        }

        if ($data['visit_type'] == VisitTypeEnum::OWNED) {
            $data['hidden_status'] = 1;
        }

        if (!$data['pid']) {
            $this->validate($request, [
                'logo' => 'required'
            ]);
        }


//        if ($data['type'] == TypeEnum::ORGANIZATION) {
//            $this->validate($request, [
//                'organization_id' => 'required|integer'
//            ]);
//        }

        try {
            $department = $this->repository->update($data, $id);
            return Response::success($department);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * Author: Mead
     */
    public function selectOptions()
    {
        $lists = $this->repository->skipPresenter()->all(['id', 'name']);
        return Response::success($lists);
    }

    /**
     * 数
     * @param Request $request
     * Author: Mead
     */
    public function trees(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $organization_id = $request->get('organization_id', false);

        $where[] = ['status', '=', ModelStatusEnum::OK];

        if ($organization_id) {
            $where[] = ['organization_id', '=', $organization_id];
        }

        $where[] = ['type', '<>', TypeEnum::PERSON];

        $id = $request->get('id');

        $trees = Category::trees($id, [
            'select' => ['id', 'name', 'pid', 'visit_type', 'logo', 'sort', 'status', 'type', 'updated_at'],
            'where' => $where,
            'sort' => 'sort desc,id desc'
        ], true);
        return Response::success($trees);
    }
}
