<?php

namespace App\Http\Controllers\Admin\Navigation;

use App\Http\Controllers\Controller;
use App\Jobs\ImportWebsitesJob;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Base\Resource;
use App\Repositories\Models\Navigation\Organization;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Navigation\OrganizationRepository;
use App\Repositories\Validators\Navigation\OrganizationValidator;

/**
 * Class OrganizationsController.
 *
 * @package namespace App\Http\Controllers\Navigation;
 */
class OrganizationsController extends Controller
{
    /**
     * @var OrganizationRepository
     */
    protected $repository;

    /**
     * @var OrganizationValidator
     */
    protected $validator;

    /**
     * OrganizationsController constructor.
     *
     * @param OrganizationRepository $repository
     * @param OrganizationValidator $validator
     */
    public function __construct(OrganizationRepository $repository, OrganizationValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));

        return Response::success($lists);
    }


    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        if (Organization::query()->where('prefix', $data['prefix'])->exists()) {
            return $this->error('域名前缀已经存在！');
        }

        if ($data['intro_title'] || $data['intro']) {
            if (!($data['intro_title'] && $data['intro'])) {
                return $this->error('底部标题和内容必须都填写！');
            }
        }

        try {
            $data['slug'] = Str::random(6);
            $data['admin_id'] = optional(login_admin())->id;
            $data['tier'] = Organization::byIdGetTier($data['pid']);
            $data['status'] = ModelStatusEnum::OK;
            $department = $this->repository->create($data);
            return Response::success($department);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $department = $this->repository->find($id);
        return Response::success($department);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
        if (Organization::query()->where('id', '<>', $id)->where('prefix', $data['prefix'])->exists()) {
            return $this->error('域名前缀已经存在！');
        }

        if ($data['intro_title'] || $data['intro']) {
            if (!($data['intro_title'] && $data['intro'])) {
                return $this->error('底部标题和内容必须都填写！');
            }
        }

        try {
            $model = $this->repository->update($request->all(), $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * Author: Mead
     */
    public function selectOptions()
    {
        $lists = $this->repository->skipPresenter()->all(['id', 'name']);
        return Response::success($lists);
    }

    /**
     * 数
     * @param Request $request
     * Author: Mead
     */
    public function trees(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');

        $trees = Organization::trees($id, ['select' => ['*']], true);
        return Response::success($trees);
    }

    /**
     * 导入网站
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function importWebsites(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'organization_id' => 'required|integer'
        ]);

        $resource = Resource::query()->where('id', $request->get('id'))->first();
        if (!$resource) {
            return $this->errorFail();
        }
        $organization_id = $request->get('organization_id', 0);
        $this->dispatch(new ImportWebsitesJob($resource['disk'], $resource['path'], $organization_id));
        return Response::success(null, '正在导入，请稍后');
    }
}
