<?php

namespace App\Http\Controllers\Admin\Qunali;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\CheckStatusEnum;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\Qunali\EventEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Qunali\Point;
use App\Repositories\Models\Qunali\PointEvent;
use App\Services\TencentMapService;
use Carbon\Carbon;
use EasyWeChat\Kernel\Support\Str;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;

use App\Contracts\Repositories\Qunali\PointRepository;
use App\Repositories\Validators\Qunali\PointValidator;

/**
 * Class PointsController.
 *
 * @package namespace App\Http\Controllers\Qunali;
 */
class PointsController extends Controller
{
    /**
     * @var PointRepository
     */
    protected $repository;

    /**
     * @var PointValidator
     */
    protected $validator;

    /**
     * PointsController constructor.
     *
     * @param PointRepository $repository
     * @param PointValidator $validator
     */
    public function __construct(PointRepository $repository, PointValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));

        return Response::success($lists);
    }


    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        $data['organization_id'] = 1;

        try {
            $data['check_status'] = CheckStatusEnum::OK;
            $data['slug'] = Str::random(8);

            $department = $this->repository->create($data);
            return Response::success($department);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $department = $this->repository->find($id);
        return Response::success($department);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
        $data['organization_id'] = 1;
        $data['check_status'] = CheckStatusEnum::OK;
        try {
            $department = $this->repository->update($data, $id);
            return Response::success($department);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * Author: Mead
     */
    public function areaSelectOptions()
    {
        $lists = Point::query()->whereNotNull('area_name')->distinct()->pluck('area_name');
        return Response::success($lists);
    }

    /**
     * 选项
     * Author: Mead
     */
    public function streetSelectOptions(Request $request)
    {
        $area_name = $request->get('area_name', false);
        $lists = Point::query()->when($area_name, function ($query) use ($area_name) {
            return $query->where('area_name', $area_name);
        })->whereNotNull('area_name')->distinct()->pluck('street_name');
        return Response::success($lists);
    }

    /**
     * 审核
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function check(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'status' => 'required|integer',
        ]);

        $id = $request->get('id');
        $status = $request->get('status');

        $point = Point::query()->where('id', $id)->first();
        if (!$point) {
            return Response::fail('找不到该数据');
        }

        if ($point->check_status != CheckStatusEnum::WAIT) {
            return Response::fail('该点已经审核过了，请勿重复操作');
        }

        switch ($status) {
            case 1:
                $old_point = Point::query()->where('slug', $point->slug)->where('check_status', CheckStatusEnum::OK)->status()->orderBy('id')->first();

                $point->version = $old_point->version + 1;
                $point->check_status = CheckStatusEnum::OK;
                $point->check_user_id = login_user_id();
                $point->check_time = Carbon::now()->toDateTimeString();
                $point->status = ModelStatusEnum::OK;
                $point->save();

                $old_point->status = ModelStatusEnum::PAUSE;
                $old_point->save();

                break;
            case 2:
                $point->check_status = CheckStatusEnum::ERROR;
                $point->check_user_id = login_user_id();
                $point->check_time = Carbon::now()->toDateTimeString();
                $point->status = ModelStatusEnum::PAUSE;
                $point->save();
                break;
            default:
                return Response::fail('数据有误');
        }
        return Response::success(null);
    }

    /**
     * 历史版本
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function history(Request $request)
    {
        $this->validate($request, [
            'slug' => 'required',
        ]);

        $slug = $request->get('slug');
        $history = Point::query()->with('user')->where('slug', $slug)->where('check_status', CheckStatusEnum::OK)->select(['id', 'name', 'time', 'user_id', 'created_at', 'version'])->orderByDesc('version')->get();
        return Response::success($history);
    }

    /**
     * 版本回退
     * @param Request $request
     * Author: Mead
     */
    public function backHistory(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);
        $id = $request->get('id');
        $old_point = Point::query()->where('id', $id)->where('check_status', CheckStatusEnum::OK)->first();
        if (!$old_point) {
            return Response::fail('找不到资源');
        }

        if ($old_point->status != ModelStatusEnum::PAUSE) {
            return Response::fail('资源状态不对');
        }

        $point = Point::query()->where('slug', $old_point->slug)->where('check_status', CheckStatusEnum::OK)->status()->orderByDesc('id')->first();
        $point->status = ModelStatusEnum::PAUSE;
        $point->save();

        $old_point->status = ModelStatusEnum::OK;
        $old_point->save();
        return Response::success(null);
    }

    /**
     * 获取地址
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function geocoder(Request $request)
    {
        $lat = $request->get('lat');
        $lng = $request->get('lng');
        $data = (new TencentMapService())->geocoder($lat, $lng);
        if (!$data) {
            return Response::fail('获取失败');
        }
        $response = [
            'province' => isset($data['ad_info']['province']) ? $data['ad_info']['province'] : null,
            'city' => isset($data['ad_info']['city']) ? $data['ad_info']['city'] : null,
            'area' => isset($data['ad_info']['district']) ? $data['ad_info']['district'] : null,
            'street' => isset($data['address_reference']['town']['title']) ? $data['address_reference']['town']['title'] : null,
            'address' => isset($data['address_reference']['crossroad']['title']) ? $data['address_reference']['crossroad']['title'] : null,
        ];
        return Response::success($response);
    }

    /**
     * 记录
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function record(Request $request)
    {
        $id = $request->get('id');
        $type = $request->get('type', 0);

        $record = PointEvent::query()->whereIn('type', [EventEnum::GOOD, EventEnum::BAD, EventEnum::REPORT])->with(['user'])->when($type, function ($query) use ($type) {
            return $query->where('type', $type);
        })->where('point_id', $id)->status()->select(['type', 'user_id', 'created_at'])->simplePaginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($record);
    }
}
