<?php

namespace App\Http\Controllers\Api\Navigation;

use App\Http\Controllers\Controller;
use Prettus\Validator\Contracts\ValidatorInterface;
use Prettus\Validator\Exceptions\ValidatorException;

use App\Contracts\Repositories\Navigation\SearchRecordRepository;
use App\Repositories\Validators\Navigation\SearchRecordValidator;

/**
 * Class SearchRecordsController.
 *
 * @package namespace App\Http\Controllers\Navigation;
 */
class SearchRecordsController extends Controller
{
    /**
     * @var SearchRecordRepository
     */
    protected $repository;

    /**
     * @var SearchRecordValidator
     */
    protected $validator;

    /**
     * SearchRecordsController constructor.
     *
     * @param SearchRecordRepository $repository
     * @param SearchRecordValidator $validator
     */
    public function __construct(SearchRecordRepository $repository, SearchRecordValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $this->repository->pushCriteria(app('Prettus\Repository\Criteria\RequestCriteria'));
        $searchRecords = $this->repository->all();

        if (request()->wantsJson()) {

            return response()->json([
                'data' => $searchRecords,
            ]);
        }

        return view('searchRecords.index', compact('searchRecords'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param SearchRecordCreateRequest $request
     *
     * @return \Illuminate\Http\Response
     *
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function store(SearchRecordCreateRequest $request)
    {
        try {

            $this->validator->with($request->all())->passesOrFail(ValidatorInterface::RULE_CREATE);

            $searchRecord = $this->repository->create($request->all());

            $response = [
                'message' => 'SearchRecord created.',
                'data' => $searchRecord->toArray(),
            ];

            if ($request->wantsJson()) {

                return response()->json($response);
            }

            return redirect()->back()->with('message', $response['message']);
        } catch (ValidatorException $e) {
            if ($request->wantsJson()) {
                return response()->json([
                    'error' => true,
                    'message' => $e->getMessageBag()
                ]);
            }

            return redirect()->back()->withErrors($e->getMessageBag())->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $searchRecord = $this->repository->find($id);

        if (request()->wantsJson()) {

            return response()->json([
                'data' => $searchRecord,
            ]);
        }

        return view('searchRecords.show', compact('searchRecord'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $searchRecord = $this->repository->find($id);

        return view('searchRecords.edit', compact('searchRecord'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param SearchRecordUpdateRequest $request
     * @param string $id
     *
     * @return Response
     *
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function update(SearchRecordUpdateRequest $request, $id)
    {
        try {

            $this->validator->with($request->all())->passesOrFail(ValidatorInterface::RULE_UPDATE);

            $searchRecord = $this->repository->update($request->all(), $id);

            $response = [
                'message' => 'SearchRecord updated.',
                'data' => $searchRecord->toArray(),
            ];

            if ($request->wantsJson()) {

                return response()->json($response);
            }

            return redirect()->back()->with('message', $response['message']);
        } catch (ValidatorException $e) {

            if ($request->wantsJson()) {

                return response()->json([
                    'error' => true,
                    'message' => $e->getMessageBag()
                ]);
            }

            return redirect()->back()->withErrors($e->getMessageBag())->withInput();
        }
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $deleted = $this->repository->delete($id);

        if (request()->wantsJson()) {

            return response()->json([
                'message' => 'SearchRecord deleted.',
                'deleted' => $deleted,
            ]);
        }

        return redirect()->back()->with('message', 'SearchRecord deleted.');
    }
}
