<?php

namespace App\Http\Controllers\Api\Navigation;

use App\Http\Controllers\Controller;
use Prettus\Validator\Contracts\ValidatorInterface;
use Prettus\Validator\Exceptions\ValidatorException;
use App\Contracts\Repositories\Navigation\UserRecordRepository;
use App\Repositories\Validators\Navigation\UserRecordValidator;

/**
 * Class UserRecordsController.
 *
 * @package namespace App\Http\Controllers\Navigation;
 */
class UserRecordsController extends Controller
{
    /**
     * @var UserRecordRepository
     */
    protected $repository;

    /**
     * @var UserRecordValidator
     */
    protected $validator;

    /**
     * UserRecordsController constructor.
     *
     * @param UserRecordRepository $repository
     * @param UserRecordValidator $validator
     */
    public function __construct(UserRecordRepository $repository, UserRecordValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $this->repository->pushCriteria(app('Prettus\Repository\Criteria\RequestCriteria'));
        $userRecords = $this->repository->all();

        if (request()->wantsJson()) {

            return response()->json([
                'data' => $userRecords,
            ]);
        }

        return view('userRecords.index', compact('userRecords'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param UserRecordCreateRequest $request
     *
     * @return \Illuminate\Http\Response
     *
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function store(UserRecordCreateRequest $request)
    {
        try {

            $this->validator->with($request->all())->passesOrFail(ValidatorInterface::RULE_CREATE);

            $userRecord = $this->repository->create($request->all());

            $response = [
                'message' => 'UserRecord created.',
                'data' => $userRecord->toArray(),
            ];

            if ($request->wantsJson()) {

                return response()->json($response);
            }

            return redirect()->back()->with('message', $response['message']);
        } catch (ValidatorException $e) {
            if ($request->wantsJson()) {
                return response()->json([
                    'error' => true,
                    'message' => $e->getMessageBag()
                ]);
            }

            return redirect()->back()->withErrors($e->getMessageBag())->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $userRecord = $this->repository->find($id);

        if (request()->wantsJson()) {

            return response()->json([
                'data' => $userRecord,
            ]);
        }

        return view('userRecords.show', compact('userRecord'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $userRecord = $this->repository->find($id);

        return view('userRecords.edit', compact('userRecord'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param UserRecordUpdateRequest $request
     * @param string $id
     *
     * @return Response
     *
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function update(UserRecordUpdateRequest $request, $id)
    {
        try {

            $this->validator->with($request->all())->passesOrFail(ValidatorInterface::RULE_UPDATE);

            $userRecord = $this->repository->update($request->all(), $id);

            $response = [
                'message' => 'UserRecord updated.',
                'data' => $userRecord->toArray(),
            ];

            if ($request->wantsJson()) {

                return response()->json($response);
            }

            return redirect()->back()->with('message', $response['message']);
        } catch (ValidatorException $e) {

            if ($request->wantsJson()) {

                return response()->json([
                    'error' => true,
                    'message' => $e->getMessageBag()
                ]);
            }

            return redirect()->back()->withErrors($e->getMessageBag())->withInput();
        }
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $deleted = $this->repository->delete($id);

        if (request()->wantsJson()) {

            return response()->json([
                'message' => 'UserRecord deleted.',
                'deleted' => $deleted,
            ]);
        }

        return redirect()->back()->with('message', 'UserRecord deleted.');
    }
}
