<?php

namespace App\Http\Controllers\Api\Navigation;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Navigation\Category;
use App\Repositories\Models\Navigation\Tag;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Navigation\WebsiteRepository;
use App\Repositories\Validators\Navigation\WebsiteValidator;

/**
 * Class WebsitesController.
 *
 * @package namespace App\Http\Controllers\Navigation;
 */
class WebsitesController extends Controller
{
    /**
     * @var WebsiteRepository
     */
    protected $repository;

    /**
     * @var WebsiteValidator
     */
    protected $validator;

    /**
     * WebsitesController constructor.
     *
     * @param WebsiteRepository $repository
     * @param WebsiteValidator $validator
     */
    public function __construct(WebsiteRepository $repository, WebsiteValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));

        return Response::success($lists);
    }


    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

//        if ($data['type'] == TypeEnum::ORGANIZATION) {
//            $data['organization_id'] = Category::query()->where('id', $data['category_id'])->value('organization_id');
//        }

        $category = Category::query()->where('id', $data['category_id'])->select(['organization_id', 'type'])->first();
        $data['organization_id'] = optional($category)->organization_id;
        $data['type'] = optional($category)->type;

        try {
            $data['user_id'] = optional(login_user())->id;
            $data['status'] = ModelStatusEnum::OK;

            $department = $this->repository->create($data);
            return Response::success($department);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $department = $this->repository->find($id);
        return Response::success($department);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
        $category = Category::query()->where('id', $data['category_id'])->select(['organization_id', 'type'])->first();
        $data['organization_id'] = optional($category)->organization_id;
        $data['type'] = optional($category)->type;

        try {
            $department = $this->repository->update($data, $id);
            return Response::success($department);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * Author: Mead
     */
    public function selectOptions()
    {
        $lists = $this->repository->skipPresenter()->all(['id', 'name']);
        return Response::success($lists);
    }

    /**
     * 数
     * @param Request $request
     * Author: Mead
     */
    public function trees(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');

        $trees = Category::trees($id, [
            'select' => ['id', 'name', 'pid', 'logo', 'sort'],
            'where' => [
                ['status', '=', ModelStatusEnum::OK]
            ],
            'sort' => 'sort desc,id desc'
        ], true);
        return Response::success($trees);
    }

    /**
     * 创建标签
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function storeTag(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
        ]);
        $name = $request->get('name');
        $tag = Tag::query()->firstOrCreate(['name' => $name], []);
        return Response::success($tag);
    }

    /**
     * 标签
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function tags(Request $request)
    {
        $tags = Tag::query()->select(['id', 'name'])->orderByDesc('id')->get();
        return Response::success($tags);
    }
}
