<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\Api\Qunali;

use App\Events\Event;
use App\Http\Controllers\Controller;
use App\Repositories\Enums\CheckStatusEnum;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\Qunali\EventEnum;
use App\Repositories\Models\Qunali\Point;
use App\Repositories\Models\Qunali\PointEvent;
use App\Services\TencentMapService;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Jiannei\Response\Laravel\Support\Facades\Response;

class HomeController extends Controller
{
    /**
     * 首页
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $key = $request->get('key');
        $lat = $request->get('lat');
        $lng = $request->get('lng');
        $keyword = $request->get('keyword', false);
        $organization_id = 1;
        $points = Point::query()->when($keyword, function ($query) use ($keyword) {
            return $query->where('name', 'like', "%{$keyword}%");
        })->where('organization_id', $organization_id)->where('check_status', CheckStatusEnum::OK)->where('status', ModelStatusEnum::OK)->selectRaw("*,round((2 * 6378.137 * ASIN(SQRT(POW( SIN( PI()*( {$lat} - lat )/ 360 ), 2 )+ COS( PI()* 29.504164 / 180 )* COS( lat * PI()/ 180 )* POW( SIN( PI()*( {$lng} - lng )/ 360 ), 2 )))) * 1000) AS distance")->orderBy('distance')->simplePaginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($points);
    }

    /**
     * 点详情
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show(Request $request)
    {
        $this->validate($request, [
            'slug' => 'required',
        ]);
        $slug = $request->get('slug');
        $point = Point::query()->with('user')->where('slug', $slug)->where('check_status', CheckStatusEnum::OK)->status()->orderBy('id')->first();
        if (!$point) {
            return Response::fail('找不到资源');
        }
        $preserve = Point::query()->with('user')->where('slug', $slug)->whereNotIn('user_id', [0])->where('check_status', CheckStatusEnum::OK)->select(['user_id'])->get();

        //记录
        $user_id = login_user_id();
        $id = $point->id;
        $type = EventEnum::VIEW;
        PointEvent::query()->create([
            'point_id' => $id,
            'user_id' => $user_id,
            'type' => $type,
            'day' => date('Y-m-d')
        ]);
        Point::query()->where('id', $id)->increment('view_nums');

        return Response::success(compact('point', 'preserve'));
    }

    /**
     * 点赞
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * Author: Mead
     */
    public function good(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');
        $user_id = login_user_id();
        $type = EventEnum::GOOD;

        $lock = Cache::lock("event:point:{$type}:{$user_id}:{$id}", 10);
        $lock->block(10);
        $is_exist = PointEvent::query()->where('user_id', $user_id)->where('point_id', $id)->where('type', $type)->exists();
        if ($is_exist) {
            $lock->release();
            return Response::fail('请勿重复操作');
        }

        PointEvent::query()->create([
            'point_id' => $id,
            'user_id' => $user_id,
            'type' => $type,
            'day' => date('Y-m-d')
        ]);
        Point::query()->where('id', $id)->increment('good_nums');
        $lock->release();

        return Response::success(null);
    }

    /**
     * 信息无用
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * Author: Mead
     */
    public function bad(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');
        $user_id = login_user_id();
        $type = EventEnum::BAD;

        $lock = Cache::lock("event:point:{$type}:{$user_id}:{$id}", 10);
        $lock->block(10);
        $is_exist = PointEvent::query()->where('user_id', $user_id)->where('point_id', $id)->where('type', $type)->exists();
        if ($is_exist) {
            $lock->release();
            return Response::fail('请勿重复操作');
        }

        PointEvent::query()->create([
            'point_id' => $id,
            'user_id' => $user_id,
            'type' => $type,
            'day' => date('Y-m-d')
        ]);
        Point::query()->where('id', $id)->increment('bad_nums');
        $lock->release();

        return Response::success(null);
    }

    /**
     * 举报
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * Author: Mead
     */
    public function report(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
//            'body' => 'required',
        ]);
        $id = $request->get('id');
//        $body = $request->get('body');
        $user_id = login_user_id();
        $type = EventEnum::REPORT;

        $lock = Cache::lock("event:point:{$type}:{$user_id}:{$id}", 10);
        $lock->block(10);
        $is_exist = PointEvent::query()->where('user_id', $user_id)->where('point_id', $id)->where('type', $type)->exists();
        if ($is_exist) {
            $lock->release();
            return Response::fail('请勿重复操作');
        }

        PointEvent::query()->create([
            'point_id' => $id,
            'user_id' => $user_id,
            'type' => $type,
//            'body' => $body,
            'day' => date('Y-m-d')
        ]);

        Point::query()->where('id', $id)->increment('report_nums');
        $lock->release();

        return Response::success(null);
    }

    /**
     * 分享
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * Author: Mead
     */
    public function share(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');
        $user_id = login_user_id();
        $type = EventEnum::SHARE;

        $lock = Cache::lock("event:point:{$type}:{$user_id}:{$id}", 10);
        $lock->block(10);
        $is_exist = PointEvent::query()->where('user_id', $user_id)->where('point_id', $id)->where('type', $type)->exists();
        if ($is_exist) {
            $lock->release();
            return Response::fail('请勿重复操作');
        }

        PointEvent::query()->create([
            'point_id' => $id,
            'user_id' => $user_id,
            'type' => $type,
            'day' => date('Y-m-d')
        ]);

        Point::query()->where('id', $id)->increment('share_nums');

        $lock->release();

        return Response::success(null);
    }


    /**
     * 编辑新版本
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function version(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'name' => 'required',
            'cover' => 'nullable',
            'address' => 'required',
            'province_name' => 'required',
            'city_name' => 'required',
            'area_name' => 'required',
            'street_name' => 'required',
            'time' => 'required',
            'lat' => 'required',
            'lng' => 'required',
        ]);

        $id = $request->get('id');
        $data = $request->only([
            'name',
            'cover',
            'address',
            'time',
            'lat',
            'lng',
            'province_name',
            'city_name',
            'area_name',
            'street_name',
        ]);

        $point = Point::query()->where('id', $id)->where('check_status', CheckStatusEnum::OK)->status()->first();

        if (!$point) {
            return Response::fail('找不到该站点');
        }


        Point::query()->create([
            'organization_id' => $point->organization_id,
            'name' => $data['name'],
            'cover' => $data['cover'],
            'address' => $data['address'],
            'province_name' => $data['province_name'],
            'city_name' => $data['city_name'],
            'area_name' => $data['area_name'],
            'street_name' => $data['street_name'],
            'time' => $data['time'],
            'lat' => $data['lat'],
            'lng' => $data['lng'],
            'slug' => $point['slug'],
            'version' => $point['version'] + 1,
//            'check_status' => CheckStatusEnum::WAIT,
            'check_status' => CheckStatusEnum::OK,
            'check_time' => Carbon::now()->toDateTimeString(),
//            'status' => ModelStatusEnum::PAUSE,
            'status' => ModelStatusEnum::OK,
            'user_id' => login_user_id()
        ]);

        $point->status = ModelStatusEnum::PAUSE;
        $point->save();

        return Response::success(null);
    }

    /**
     * 审核
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function check(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'status' => 'required|integer',
        ]);

        $id = $request->get('id');
        $status = $request->get('status');

        $point = Point::query()->where('id', $id)->first();
        if (!$point) {
            return Response::fail('找不到该数据');
        }

        if ($point->check_status != CheckStatusEnum::WAIT) {
            return Response::fail('该点已经审核过了，请勿重复操作');
        }

        switch ($status) {
            case 1:
                $old_point = Point::query()->where('slug', $point->slug)->where('check_status', CheckStatusEnum::OK)->status()->orderBy('id')->first();

                $point->version = $old_point->version + 1;
                $point->check_status = CheckStatusEnum::OK;
                $point->check_user_id = login_user_id();
                $point->check_time = Carbon::now()->toDateTimeString();
                $point->status = ModelStatusEnum::OK;
                $point->save();

                $old_point->status = ModelStatusEnum::PAUSE;
                $old_point->save();

                break;
            case 2:
                $point->check_status = CheckStatusEnum::ERROR;
                $point->check_user_id = login_user_id();
                $point->check_time = Carbon::now()->toDateTimeString();
                $point->status = ModelStatusEnum::PAUSE;
                $point->save();
                break;
            default:
                return Response::fail('数据有误');
        }
        return Response::success(null);
    }

    /**
     * 点击新点
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function storePoint(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'cover' => 'nullable',
            'address' => 'required',
            'province_name' => 'required',
            'city_name' => 'required',
            'area_name' => 'required',
            'street_name' => 'required',
            'time' => 'required',
            'lat' => 'required',
            'lng' => 'required',
        ]);

        $data = $request->only([
            'name',
            'cover',
            'address',
            'time',
            'lat',
            'lng',
            'province_name',
            'city_name',
            'area_name',
            'street_name',
        ]);


        Point::query()->create([
            'organization_id' => 1,
            'name' => $data['name'],
            'cover' => $data['cover'],
            'address' => $data['address'],
            'province_name' => $data['province_name'],
            'city_name' => $data['city_name'],
            'area_name' => $data['area_name'],
            'street_name' => $data['street_name'],
            'time' => $data['time'],
            'lat' => $data['lat'],
            'lng' => $data['lng'],
            'slug' => Str::random(8),
            'version' => 1,
            'check_status' => CheckStatusEnum::OK,
            'status' => ModelStatusEnum::OK,
            'user_id' => login_user_id(),
        ]);

        return Response::success(null);
    }

    /**
     * 历史版本
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function history(Request $request)
    {
        $this->validate($request, [
            'slug' => 'required',
        ]);

        $slug = $request->get('slug');
        $history = Point::query()->with('user')->where('slug', $slug)->where('check_status', CheckStatusEnum::OK)->select(['id', 'name', 'time', 'user_id', 'created_at', 'version'])->orderByDesc('version')->get();
        return Response::success($history);
    }

    /**
     * 版本回退
     * @param Request $request
     * Author: Mead
     */
    public function backHistory(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);
        $id = $request->get('id');
        $old_point = Point::query()->where('id', $id)->where('check_status', CheckStatusEnum::OK)->first();
        if (!$old_point) {
            return Response::fail('找不到资源');
        }

        if ($old_point->status != ModelStatusEnum::PAUSE) {
            return Response::fail('资源状态不对');
        }

        $point = Point::query()->where('slug', $old_point->slug)->where('check_status', CheckStatusEnum::OK)->status()->orderByDesc('id')->first();
        $point->status = ModelStatusEnum::PAUSE;
        $point->save();

        $old_point->status = ModelStatusEnum::OK;
        $old_point->save();
        return Response::success(null);
    }

    /**
     * 获取地址
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function geocoder(Request $request)
    {
        $lat = $request->get('lat');
        $lng = $request->get('lng');
        $data = (new TencentMapService())->geocoder($lat, $lng);
        if (!$data) {
            return Response::fail('获取失败');
        }

        $response = [
            'province' => isset($data['ad_info']['province']) ? $data['ad_info']['province'] : null,
            'city' => isset($data['ad_info']['city']) ? $data['ad_info']['city'] : null,
            'area' => isset($data['ad_info']['district']) ? $data['ad_info']['district'] : null,
            'street' => isset($data['address_reference']['town']['title']) ? $data['address_reference']['town']['title'] : null,
            'address' => isset($data['address_reference']['crossroad']['title']) ? $data['address_reference']['crossroad']['title'] : null,
        ];
        return Response::success($response);
    }

    /**
     * 记录
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function record(Request $request)
    {
        $id = $request->get('id');
        $type = $request->get('type', 0);

        $record = PointEvent::query()->whereIn('type', [EventEnum::GOOD, EventEnum::BAD, EventEnum::REPORT])->with(['user'])->when($type, function ($query) use ($type) {
            return $query->where('type', $type);
        })->where('point_id', $id)->status()->select(['type', 'user_id', 'created_at'])->orderByDesc('id')->simplePaginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($record);
    }

}
