<?php

namespace App\Http\Controllers\Qunali;

use Illuminate\Http\Request;

use App\Http\Requests;
use Prettus\Validator\Contracts\ValidatorInterface;
use Prettus\Validator\Exceptions\ValidatorException;
use App\Http\Requests\PointEventCreateRequest;
use App\Http\Requests\PointEventUpdateRequest;
use App\Contracts\Repositories\Qunali\PointEventRepository;
use App\Repositories\Validators\Qunali\PointEventValidator;

/**
 * Class PointEventsController.
 *
 * @package namespace App\Http\Controllers\Qunali;
 */
class PointEventsController extends Controller
{
    /**
     * @var PointEventRepository
     */
    protected $repository;

    /**
     * @var PointEventValidator
     */
    protected $validator;

    /**
     * PointEventsController constructor.
     *
     * @param PointEventRepository $repository
     * @param PointEventValidator $validator
     */
    public function __construct(PointEventRepository $repository, PointEventValidator $validator)
    {
        $this->repository = $repository;
        $this->validator  = $validator;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $this->repository->pushCriteria(app('Prettus\Repository\Criteria\RequestCriteria'));
        $pointEvents = $this->repository->all();

        if (request()->wantsJson()) {

            return response()->json([
                'data' => $pointEvents,
            ]);
        }

        return view('pointEvents.index', compact('pointEvents'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  PointEventCreateRequest $request
     *
     * @return \Illuminate\Http\Response
     *
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function store(PointEventCreateRequest $request)
    {
        try {

            $this->validator->with($request->all())->passesOrFail(ValidatorInterface::RULE_CREATE);

            $pointEvent = $this->repository->create($request->all());

            $response = [
                'message' => 'PointEvent created.',
                'data'    => $pointEvent->toArray(),
            ];

            if ($request->wantsJson()) {

                return response()->json($response);
            }

            return redirect()->back()->with('message', $response['message']);
        } catch (ValidatorException $e) {
            if ($request->wantsJson()) {
                return response()->json([
                    'error'   => true,
                    'message' => $e->getMessageBag()
                ]);
            }

            return redirect()->back()->withErrors($e->getMessageBag())->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $pointEvent = $this->repository->find($id);

        if (request()->wantsJson()) {

            return response()->json([
                'data' => $pointEvent,
            ]);
        }

        return view('pointEvents.show', compact('pointEvent'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $pointEvent = $this->repository->find($id);

        return view('pointEvents.edit', compact('pointEvent'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  PointEventUpdateRequest $request
     * @param  string            $id
     *
     * @return Response
     *
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function update(PointEventUpdateRequest $request, $id)
    {
        try {

            $this->validator->with($request->all())->passesOrFail(ValidatorInterface::RULE_UPDATE);

            $pointEvent = $this->repository->update($request->all(), $id);

            $response = [
                'message' => 'PointEvent updated.',
                'data'    => $pointEvent->toArray(),
            ];

            if ($request->wantsJson()) {

                return response()->json($response);
            }

            return redirect()->back()->with('message', $response['message']);
        } catch (ValidatorException $e) {

            if ($request->wantsJson()) {

                return response()->json([
                    'error'   => true,
                    'message' => $e->getMessageBag()
                ]);
            }

            return redirect()->back()->withErrors($e->getMessageBag())->withInput();
        }
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int $id
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $deleted = $this->repository->delete($id);

        if (request()->wantsJson()) {

            return response()->json([
                'message' => 'PointEvent deleted.',
                'deleted' => $deleted,
            ]);
        }

        return redirect()->back()->with('message', 'PointEvent deleted.');
    }
}
